/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj;

import polyglot.types.Type;

public final class SJConstants
{
	public static final boolean SJ_DEBUG_MODE = false;
	public static final boolean SJ_TYPES_CHECKED = true; // Includes some translation work.

	public static final String JAVA_STRING_CLASS = "java.lang.String";
	public static final String JAVA_EXCEPTION_CLASS = "java.lang.Exception";

	public static final String SJPROTOCOL_CLASS = "sj.runtime.SJProtocol";
	public static final String SJCHANNEL_CLASS = "sj.runtime.SJServerAddress";
	public static final String SJSOCKET_CLASS = "sj.runtime.net.SJSocket";
	public static final String SJSERVER_CLASS = "sj.runtime.net.SJServerSocket";
	public static final String SJLABEL_CLASS = "java.lang.String"; // Should be changed to use the actual SJLabel class.
	public static final String SJTHREAD_CLASS = "sj.runtime.SJThread";
	//public static final String SJEXCEPTION_CLASS = "sj.runtime.SJException";
	public static String SJRUNTIMEEXCEPTION_CLASS = "sj.runtime.SJRuntimeException";

	// Initialised by the parser (first pass).
	public static Type JAVA_EXCEPTION_TYPE;

	public static Type SJPROTOCOL_TYPE;
	public static Type SJCHANNEL_TYPE;
	public static Type SJSOCKET_TYPE;
	public static Type SJSERVER_TYPE;
	public static Type SJLABEL_TYPE;
	public static Type SJTHREAD_TYPE;
	//public static Type SJEXCEPTION_TYPE;
	public static Type SJRUNTIMEEXCEPTION_TYPE;

	public static final String SJ_KEYWORD_PROTOCOL = "protocol";
	public static final String SJ_KEYWORD_CHANNEL = "serveraddress";
	public static final String SJ_KEYWORD_REC = "rec";

	public static final String SJ_KEYWORD_BEGIN = "begin";
	public static final String SJ_KEYWORD_SEND = "ssend"; // Need e.g. ssendint?
	public static final String SJ_KEYWORD_RECV = "srecv";
	public static final String SJ_KEYWORD_OBRA = "sobra";
	public static final String SJ_KEYWORD_IBRA = "sibra";
	public static final String SJ_KEYWORD_OWHI = "sowhi";
	public static final String SJ_KEYWORD_IWHI = "siwhi";
	public static final String SJ_KEYWORD_NREC = "srec";
	public static final String SJ_KEYWORD_PROT = "sprot";

	//public static final String SJ_KEYWORD_SEND = "send";
	public static final String SJ_KEYWORD_SENDBOOLEAN = "sendBoolean";
	public static final String SJ_KEYWORD_SENDINT = "sendInt";
	public static final String SJ_KEYWORD_SENDSERVERADDRESS = "sendServerAddress";
	public static final String SJ_KEYWORD_SENDSESSION = "sendSession";
	//public static final String SJ_KEYWORD_RECEIVE = "receive";
	public static final String SJ_KEYWORD_RECEIVEBOOLEAN = "receiveBoolean";
	public static final String SJ_KEYWORD_RECEIVEINT = "receiveInt";
	public static final String SJ_KEYWORD_RECEIVESERVERADDRESS = "receiveServerAddress";
	public static final String SJ_KEYWORD_RECEIVESESSION = "receiveSession";
	public static final String SJ_KEYWORD_OUTBRANCH = "outbranch";
	public static final String SJ_KEYWORD_INBRANCH = "inbranch";
	public static final String SJ_KEYWORD_OUTWHILE = "outwhile";
	public static final String SJ_KEYWORD_INWHILE = "inwhile";
	public static final String SJ_KEYWORD_RECURSION = "recursion";
	//public static final String SJ_KEYWORD_RECURSE = "recurse";
	public static final String SJ_KEYWORD_SESSREC = "sessrec";
	public static final String SJ_KEYWORD_SPAWN = "spawn";

	// Used by SJSessionOperationParser.
	public static final String SJ_SYNTAX_CREATE = "create";
	public static final String SJ_SYNTAX_ACCEPT = "accept";
	public static final String SJ_SYNTAX_REQUEST = "request";
	public static final String SJ_SYNTAX_SEND = "send";
	public static final String SJ_SYNTAX_SENDBOOLEAN= "sendBoolean";
	public static final String SJ_SYNTAX_SENDINT = "sendInt";
	public static final String SJ_SYNTAX_SENDBYTE = "sendByte";
	public static final String SJ_SYNTAX_SENDCHANNEL = "sendServerAddress";
	public static final String SJ_SYNTAX_SENDSESSION = "sendSession";
	public static final String SJ_SYNTAX_RECEIVE = "receive";
	public static final String SJ_SYNTAX_RECEIVEBOOLEAN = "receiveBoolean";
	public static final String SJ_SYNTAX_RECEIVEINT = "receiveInt";
	public static final String SJ_SYNTAX_RECEIVEBYTE = "receiveByte";
	public static final String SJ_SYNTAX_RECURSE = "recurse";
	public static final String SJ_SYNTAX_RECEIVECHANNEL = "receiveServerAddress";
	public static final String SJ_SYNTAX_RECEIVESESSION = "receiveSession";

	// Translation targets.
	public static final String SJCHANNEL_CREATE = "create";

	public static final String SJSOCKET_CREATE = "create";
	public static final String SJSOCKET_REQUEST = "request";
	public static final String SJSOCKET_CLOSE = "close";
	public static final String SJSOCKET_SEND = "send";
	public static final String SJSOCKET_SENDBOOLEAN = "sendBoolean";
	public static final String SJSOCKET_SENDINT = "sendInt";
	public static final String SJSOCKET_SENDBYTE = "sendByte";
	public static final String SJSOCKET_SENDCHANNEL = "sendServerAddress";
	public static final String SJSOCKET_SENDSESSION = "sendSession";
	public static final String SJSOCKET_RECEIVE = "receive";
	public static final String SJSOCKET_RECEIVEBOOLEAN = "receiveBoolean";
	public static final String SJSOCKET_RECEIVEINT = "receiveInt";
	public static final String SJSOCKET_RECEIVEBYTE = "receiveByte";
	public static final String SJSOCKET_RECEIVECHANNEL = "receiveServerAddress";
	public static final String SJSOCKET_RECEIVESESSION = "receiveSession";

	public static final String SJSOCKET_OUTLABEL = "outlabel";
	public static final String SJSOCKET_INLABEL = "inlabel";
	public static final String SJSOCKET_OUTSYNC = "outsync";
	public static final String SJSOCKET_INSYNC = "insync";
	public static final String SJSOCKET_RECURSIONFALSE = "recursionFalse";
	public static final String SJSOCKET_RECURSIONTRUE = "recursionTrue";

	public static final String SJSOCKET_RECURSE = "recurse"; // Only to satisfy base type checking before translation.

	/*public static final String SJSOCKET_SUSPEND = "suspend";
	public static final String SJSOCKET_RESUME = "resume";*/

	public static final String SJSERVER_CREATE = "create";
	public static final String SJSERVER_ACCEPT = "accept";

	public static final String SJ_STRING_DECL_PREFIX = "_sjprotocol_";
	public static final String SJ_INBRANCH_LABEL_FIELD_PREFIX = "_sjbranch_";
	public static final String SJ_FORMAL_PREFIX = "_sjformal_";
	public static final String SJ_RETURN_TYPE_PREFIX = "_sjreturntype_";
	public static final String SJ_RECURSION_PREFIX = "_sjrecursion_";
	public static final String SJ_SPAWN_START = "_sjstart";
	public static final String SJ_THREAD_RUNEXCFORMAL = "_sjx";
	public static final String SJ_SESSION_TRY_RETURNED = "_sjtryreturned_";

	public static final String SJLABEL_EQUALITY_TEST = "equals";

	/* // Used by SJHOSParser and SJTypeChecker in conformance with SJSocket.
	public static final int SJSOCKET_SENDSESSION_SOCKET_ARGNUM = 0;
	public static final int SJSOCKET_SENDSESSION_TYPESTRING_ARGNUM = 1;
	//public static final int SJSOCKET_RECEIVESESSION_TYPESTRING_ARGNUM = 0;*/

	public static final String POLYGLOT_TYPEDECODER_NAME_ARG = ""; // Don't know what this argument is for. Think it's just for error reporting (setting the message for a nested exception).
	public static final String SJSOCKET_DECODESJTYPE = "decodeSJType";

	public static final String SJTHREAD_RUN = "run";

	public static final String SJ_SPAWN_COUNT_PREFIX = "_sjspawncount_";
	public static final int SJ_SPAWN_COUNT_INIT = 0;
	public static final int SJ_SPAWN_COUNT_CLOSE = -1; // Should be less than the init. value.
	public static final int SJ_SPAWN_NO_CLOSE = -2; // Currently unused.
	public static final String SJSOCKET_GETSPAWNCOUNT = "getSpawnCount";
	public static final String SJSOCKET_INCREMENTSPAWNCOUNT = "incrementSpawnCount";

	// Used for printing type objects and nodes. Not suitable to declare as first-class parsing tokens because symbols already used for base language.
	public static final String SJ_STRING_SEPARATOR = ".";
	public static final String SJ_STRING_BEGIN = "begin";
	public static final String SJ_STRING_SEND_OPEN = "!<";
	public static final String SJ_STRING_SEND_CLOSE = ">";
	public static final String SJ_STRING_RECEIVE_OPEN = "?(";
	public static final String SJ_STRING_RECEIVE_CLOSE = ")";
	public static final String SJ_STRING_OUTBRANCH_OPEN = "!{";
	public static final String SJ_STRING_OUTBRANCH_CLOSE = "}";
	public static final String SJ_STRING_INBRANCH_OPEN = "?{";
	public static final String SJ_STRING_INBRANCH_CLOSE = "}";
	public static final String SJ_STRING_LABEL = ":";
	public static final String SJ_STRING_CASE_SEPARATOR = ",";
	public static final String SJ_STRING_OUTWHILE_OPEN = "![";
	public static final String SJ_STRING_OUTWHILE_CLOSE = "]*";
	public static final String SJ_STRING_INWHILE_OPEN = "?["; // '*' avoids clash with array types
	public static final String SJ_STRING_INWHILE_CLOSE = "]*";
	public static final String SJ_STRING_RECURSION_OPEN = "[";
	public static final String SJ_STRING_RECURSION_CLOSE = "]";
	public static final String SJ_STRING_REC = SJ_KEYWORD_REC;
	public static final String SJ_STRING_RECURSE_PREFIX = "#";
	public static final String SJ_STRING_UNKNOWN_TYPE = "SJUnknownType";
	public static final String SJ_STRING_META_TYPE_PREFIX = "@";

	private SJConstants() { } // Prevent instantiation
}
