/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.ast.typenodes;

import java.util.Iterator;
import java.util.List;

import polyglot.util.Position;

import sj.util.SJLabel;

import static sj.SJConstants.*;

public class SJInbranchNode_c extends SJTypeNode_c implements SJInbranchNode
{
	private List<SJBranchCaseNode> branchCases;

	public SJInbranchNode_c(Position pos, List<SJBranchCaseNode> branchCases)
	{
		super(pos);

		this.branchCases = branchCases;
	}

	public List<SJBranchCaseNode> branchCases()
	{
		return branchCases;
	}

	public SJInbranchNode branchCases(List<SJBranchCaseNode> branchCases)
	{
		this.branchCases = branchCases;

		return this;
	}

	public String nodeToString()
	{
		String s = SJ_STRING_INBRANCH_OPEN;

		for (Iterator<SJBranchCaseNode> i = branchCases().iterator(); i.hasNext(); )
		{
			SJBranchCaseNode branchCase = i.next();
			SJLabel lab = branchCase.label();
			SJTypeNode sjTypeNode = branchCase.sjTypeNode();

			s = s + lab + SJ_STRING_LABEL;

			if (sjTypeNode == null)
			{
				s = s + " ";
			}
			else
			{
				s = s + sjTypeNode.treeToString();
			}

			if (i.hasNext()) s = s + SJ_STRING_CASE_SEPARATOR + " ";
		}

		return s + SJ_STRING_INBRANCH_CLOSE;
	}
}
