/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.types;

import java.util.List;

import polyglot.types.*;
import polyglot.util.Position;

import sj.types.typeobjects.sesstypes.*;
import sj.types.typeobjects.typeableinstances.*;
import sj.util.*;

public interface SJTypeSystem extends TypeSystem
{
	public SJBeginType SJBeginType();
	public SJSendType SJSendType(Type messageType) throws SemanticException;
	public SJReceiveType SJReceiveType(Type messageType) throws SemanticException;
	public SJOutbranchType SJOutbranchType();
	public SJInbranchType SJInbranchType();
	public SJOutwhileType SJOutwhileType();
	public SJInwhileType SJInwhileType();
	public SJRecursionType SJRecursionType(SJLabel lab);
	public SJRecurseType SJRecurseType(SJLabel lab);
	public SJUnknownType SJUnknownType();

	public SJTypeableInstance SJTypeableInstance(Position pos, SJType sjType);
	public SJNamedInstance SJNamedInstance(Position pos, SJType sjType, String name);
	public SJSessionTryInstance SJSessionTryInstance(Position pos, SJMap sockets);
	public SJSessionOpInstance SJSessionOpInstance(Position pos, SJType sjType, String name);
	public SJMethodInstance SJMethodInstance(String name, SJMap sjFormalTypes, SJType sjReturnType);
	public SJRecursiveMethodInstance SJRecursiveMethodInstance(String name, SJMap sjFormalTypes, SJType sjReturnType);
	public SJThreadInstance SJThreadInstance(String name, List<SJType> runTypes);

	public boolean wellFormedRecursions(SJType sjType);
}
