/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.types.typeobjects.sesstypes;

import polyglot.types.*;

import static sj.SJConstants.SJ_STRING_INWHILE_OPEN;
import static sj.SJConstants.SJ_STRING_INWHILE_CLOSE;

public class SJInwhileType_c extends SJType_c implements SJInwhileType
{
	public static final long serialVersionUID = 1L;

	private SJType body;

	public SJInwhileType_c(TypeSystem ts)
	{
		super(ts);
	}

	public SJType body()
	{
		return (body == null) ? null : body.treeClone();
	}

	public SJInwhileType body(SJType body)
	{
		if (body == null)
		{
			this.body = null;
		}
		else
		{
			body = body.treeClone(); // Defensive copy.

			this.body = body;

			((SJType_c) body).parent(this);
		}

		return this;
	}

	public boolean nodeEquals(SJType sjType)
	{
		if (sjType == null || !(sjType instanceof SJInwhileType))
		{
			return false;
		}

		if (body() == null)
		{
			return ((SJInwhileType) sjType).body() == null;
		}
		else
		{
			return body().treeEquals(((SJInwhileType) sjType).body()); // Using treeEquals directly rather than typeEquals.
		}
	}

	public boolean nodeSubtype(SJType sjType)
	{
		if (sjType == null || !(sjType instanceof SJInwhileType))
		{
			return false;
		}

		SJType theirs = ((SJInwhileType) sjType).body();

		if (body() == null)
		{
			return theirs == null;
		}
		else
		{
			return body().treeSubtype(theirs);
		}
	}

	public boolean nodeDualtype(SJType sjType)
	{
		if (sjType == null || !(sjType instanceof SJOutwhileType))
		{
			return false;
		}

		SJType theirs = ((SJOutwhileType) sjType).body();

		if (body() == null)
		{
			return theirs == null;
		}
		else
		{
			return body().treeDualtype(theirs);
		}
	}

	public SJType nodeSubsume(SJType sjType) throws SemanticException
	{
		if (!(sjType instanceof SJInwhileType))
		{
			throw new SemanticException("Not subsumable: " + this + ", " + sjType);
		}

		SJType ours = body();
		SJType theirs = ((SJInwhileType) sjType).body();

		SJInwhileType res = typeSystem().SJInwhileType();

		if (ours == null)
		{
			if (theirs != null)
			{
				throw new SemanticException("Not subsumable: " + this + ", " + sjType);
			}
		}
		else
		{
			if (theirs == null)
			{
				throw new SemanticException("Not subsumable: " + this + ", " + sjType);
			}

			res = res.body(ours.subsume(theirs));
		}

		return res;
	}

	public boolean nodeWellFormed()
	{
		return (body() == null) ? true : body().treeWellFormed();
	}

	public SJType nodeClone()
	{
		if (body() == null)
		{
			return typeSystem().SJInwhileType();
		}
		else
		{
			return typeSystem().SJInwhileType().body((SJType) body().treeClone());
		}
	}

	public String nodeToString()
	{
		if (body == null)
		{
			return SJ_STRING_INWHILE_OPEN + SJ_STRING_INWHILE_CLOSE;
		}
		else
		{
			return SJ_STRING_INWHILE_OPEN + body.toString() + SJ_STRING_INWHILE_CLOSE;
		}
	}
}
