/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.types.typeobjects.sesstypes;

import polyglot.types.*;

import sj.util.SJCompilerUtils;

import static sj.SJConstants.SJ_STRING_RECEIVE_OPEN;
import static sj.SJConstants.SJ_STRING_RECEIVE_CLOSE;

public class SJReceiveType_c extends SJType_c implements SJReceiveType
{
	public static final long serialVersionUID = 1L;

	private Type messageType;

	//private long svuid = -1; // Factor out constant.

	public SJReceiveType_c(TypeSystem ts, Type messageType) throws SemanticException
	{
		super(ts);

		messageType(messageType);
	}

	public Type messageType()
	{
		return messageType;
	}

	public SJType messageType(Type messageType) throws SemanticException
	{
		if (messageType instanceof SJType)
		{
			messageType = ((SJType) messageType).treeClone(); // Only session types cloned - pointer equality maintained for ordinary types.
		}
		else
		{
			//this.svuid = SJCompilerUtils.getSVUID(messageType);
		}

		this.messageType = messageType; // No need to clone?

		return this;
	}

	public boolean nodeEquals(SJType sjType)
	{
		return (sjType == null) ? false : (sjType instanceof SJReceiveType) && messageType().typeEquals(((SJReceiveType) sjType).messageType());
	}

	public boolean nodeSubtype(SJType sjType)
	{
		return (sjType == null) ? false : (sjType instanceof SJReceiveType) && messageType().isSubtype(((SJReceiveType) sjType).messageType()); // "Reversed" so that...
	}

	public boolean nodeDualtype(SJType sjType) {
		return (sjType == null) ? false : (sjType instanceof SJSendType) && ((SJSendType) sjType).messageType().isSubtype(messageType()); // ...higher-order variance can be treated uniformly with ordinary messages types.
	}

	public SJType nodeSubsume(SJType sjType) throws SemanticException
	{
		if (!(sjType instanceof SJReceiveType))
		{
			throw new SemanticException("Not subsumable: " + this + ", " + sjType);
		}

		return typeSystem().SJReceiveType(SJCompilerUtils.subsumeReceiveTypes(messageType(), ((SJReceiveType) sjType).messageType()));
	}

	public boolean nodeWellFormed()
	{
		Type type = messageType();

		if (type instanceof SJBeginType)
		{
			return ((SJBeginType) type).wellFormed();
		}
		else if (type instanceof SJType)
		{
			return ((SJType) type).treeWellFormed();
		}
		else
		{
			return true;
		}
	}

	public SJType nodeClone()
	{
		try
		{
			return typeSystem().SJReceiveType(messageType()); // No need to clone message type objects?
		}
		catch (SemanticException se)
		{
			// Not possible - any problems would have been raised when this was orig. created.
			throw new RuntimeException(se);
		}
	}

	public String nodeToString()
	{
		String message = messageType.toString(); // toString enough for messageType? or need to manually get full name?

		return SJ_STRING_RECEIVE_OPEN + message + SJ_STRING_RECEIVE_CLOSE;
	}
}
