/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.types.typeobjects.sesstypes;

import polyglot.types.*;

import sj.util.SJLabel;

import static sj.SJConstants.*;

public class SJRecurseType_c extends SJType_c implements SJRecurseType
{
	public static final long serialVersionUID = 1L;

	private SJLabel lab;

	public SJRecurseType_c(TypeSystem ts, SJLabel lab)
	{
		super(ts);

		this.lab = lab;
	}

	public SJLabel label()
	{
		return lab;
	}

	public SJRecurseType label(SJLabel lab)
	{
		this.lab = lab;

		return this;
	}

	public boolean nodeEquals(SJType sjType)
	{
		if (sjType == null || !(sjType instanceof SJRecurseType))
		{
			return false;
		}

		return label().equals(((SJRecurseType) sjType).label());
	}

	public boolean nodeSubtype(SJType sjType)
	{
		if (sjType == null || !(sjType instanceof SJRecurseType))
		{
			return false;
		}

		return label().equals(((SJRecurseType) sjType).label());
	}

	public boolean nodeDualtype(SJType sjType)
	{
		if (sjType == null || !(sjType instanceof SJRecurseType))
		{
			return false;
		}

		return label().equals(((SJRecurseType) sjType).label());
	}

	public SJType nodeSubsume(SJType sjType) throws SemanticException
	{
		if (!(sjType instanceof SJRecurseType))
		{
			throw new SemanticException("Not subsumable: " + this + ", " + sjType);
		}

		SJLabel ours = label();
		SJLabel theirs = ((SJRecurseType) sjType).label();

		if (!ours.equals(theirs))
		{
			throw new SemanticException("Not subsumable: " + this + ", " + sjType);
		}

		return typeSystem().SJRecurseType(ours);
	}

	public boolean nodeWellFormed() // Well-formed recursions checked by external routine.
	{
		return true;
	}

	public SJType nodeClone()
	{
		return typeSystem().SJRecurseType(label().clone());
	}

	public String nodeToString()
	{
		return SJ_STRING_RECURSE_PREFIX + label().toString();
	}
}
