/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.types.typeobjects.sesstypes;

import polyglot.types.*;

import sj.util.SJLabel;

import static sj.SJConstants.*;

public class SJRecursionType_c extends SJType_c implements SJRecursionType
{
	public static final long serialVersionUID = 1L;

	private SJLabel lab;
	private SJType body;

	public SJRecursionType_c(TypeSystem ts, SJLabel lab)
	{
		super(ts);

		this.lab = lab;
	}

	public SJLabel label()
	{
		return lab;
	}

	public SJRecursionType label(SJLabel lab)
	{
		this.lab = lab;

		return this;
	}

	public SJType body()
	{
		return (body == null) ? null : body.treeClone();
	}

	public SJRecursionType body(SJType body)
	{
		if (body == null)
		{
			this.body = null;
		}
		else
		{
			body = body.treeClone(); // Defensive copy.

			this.body = body;

			((SJType_c) body).parent(this);
		}

		return this;
	}

	public boolean nodeEquals(SJType sjType)
	{
		if (sjType == null || !(sjType instanceof SJRecursionType))
		{
			return false;
		}

		SJRecursionType them = (SJRecursionType) sjType;

		if (!label().equals(them.label()))
		{
			return false;
		}

		if (body() == null)
		{
			return them.body() == null;
		}
		else
		{
			return body().treeEquals(them.body());
		}
	}

	public boolean nodeSubtype(SJType sjType)
	{
		if (sjType == null || !(sjType instanceof SJRecursionType))
		{
			return false;
		}

		SJRecursionType them = (SJRecursionType) sjType;

		if (!label().equals(them.label()))
		{
			return false;
		}

		if (body() == null)
		{
			return them.body() == null;
		}
		else
		{
			return body().treeSubtype(them.body());
		}
	}

	public SJType nodeSubsume(SJType sjType) throws SemanticException
	{
		if (!(sjType instanceof SJRecursionType))
		{
			throw new SemanticException("Not subsumable: " + this + ", " + sjType);
		}

		SJRecursionType them = (SJRecursionType) sjType;

		if (!label().equals(them.label()))
		{
			throw new SemanticException("Not subsumable: " + this + ", " + sjType);
		}

		SJType ours = body();
		SJType theirs = them.body();

		SJRecursionType res = typeSystem().SJRecursionType(label());

		if (ours == null)
		{
			if (theirs != null)
			{
				throw new SemanticException("Not subsumable: " + this + ", " + sjType);
			}
		}
		else
		{
			if (theirs == null)
			{
				throw new SemanticException("Not subsumable: " + this + ", " + sjType);
			}

			res = res.body(ours.subsume(theirs));
		}

		return res;
	}

	public boolean nodeDualtype(SJType sjType)
	{
		if (sjType == null || !(sjType instanceof SJRecursionType))
		{
			return false;
		}

		SJRecursionType them = (SJRecursionType) sjType;

		if (!label().equals(them.label()))
		{
			return false;
		}

		if (body() == null)
		{
			return them.body() == null;
		}
		else
		{
			return body().treeDualtype(them.body());
		}
	}

	public boolean nodeWellFormed() // Well-formed recursions checked by external routine.
	{
		return (body() == null) ? true : body().treeWellFormed();
	}

	public SJType nodeClone()
	{
		if (body() == null)
		{
			return typeSystem().SJRecursionType(label().clone());
		}
		else
		{
			return typeSystem().SJRecursionType(label().clone()).body(body().treeClone());
		}
	}

	public String nodeToString()
	{
		String m = SJ_STRING_REC + " " + label() + SJ_STRING_RECURSION_OPEN;

		if (body() == null)
		{
			return m + SJ_STRING_RECURSION_CLOSE;
		}
		else
		{
			return m + body().toString() + SJ_STRING_RECURSION_CLOSE;
		}
	}
}
