/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.types.typeobjects.sesstypes;

import polyglot.types.SemanticException;
import polyglot.types.*;

import sj.types.*;

import static sj.SJConstants.SJ_STRING_SEPARATOR;

public abstract class SJType_c extends Type_c implements SJType // Doesn't inherit any code from SJTypeObject_c (no new code there anyway*).
{
	public static final long serialVersionUID = 1L;

	private SJType child;
	private SJType parent; // This is probably not useful.

	protected SJType_c()
	{
		super();
	}

	public SJType_c(TypeSystem ts)
	{
		super(ts);
	}

	public SJTypeSystem typeSystem() // * Except for this.
	{
		return (SJTypeSystem) super.typeSystem();
	}

	public SJType child()
	{
		return (child == null) ? null : child.treeClone();
	}

	public SJType child(SJType child)
	{
		this.child = child.treeClone();

		//return this;
		return ((SJType_c) child()).parent(this); // Clone would cycle forever (regardless of setting parent).
	}

	public SJType parent()
	{
		return parent; // No clone, ancestors would be lost.
	}

	protected SJType parent(SJType parent)
	{
		this.parent = parent; // No clone (would cycle forever with child).

		return parent;
	}

	private SJType leaf()
	{
		if (child == null)
		{
			return this;
		}
		else
		{
			return ((SJType_c) child).leaf(); // No defensive copy, so private.
		}
	}

	public SJType append(SJType sjt)
	{
		if (sjt != null)
		{
			leaf().child(sjt.treeClone());//.treeClone();
		}

		return this; // No clone (like child method).
	}

	// Parent isSubtype method uses typeEquals and descendsFrom.
	public boolean typeEqualsImpl(Type t)
	{
		//return super.typeEqualsImpl(t); // Pointer equality. Later change this to type equivalence.
		return (t instanceof SJType) ? treeEquals((SJType) t) : false;
	}

	public boolean descendsFromImpl(Type t) // Not quite uniform with Polyglot: descendsFrom should be strict but treeSubtype (because nodeSubtype) also permits equality. So actually isSubtype doesn't really need to call typeEquals here.
	{
		return (t instanceof SJType) ? treeSubtype((SJType) t) : false;
	}

	public boolean isDualtype(Type t)
	{
		return (t instanceof SJType) ? treeDualtype((SJType) t) : false;
	}

	public boolean treeEquals(SJType sjTree)
	{
		return nodeEquals(sjTree) ? ((child() == null) ? ((sjTree.child() == null) ? true : false) : child().treeEquals(sjTree.child())) : false;
	}

	public boolean treeSubtype(SJType sjTree)
	{
		return nodeSubtype(sjTree) ? ((child() == null) ? ((sjTree.child() == null) ? true : false) : child().treeSubtype(sjTree.child())) : false;
	}

	public boolean treeDualtype(SJType sjTree)
	{
		return nodeDualtype(sjTree) ? ((child() == null) ? ((sjTree.child() == null) ? true : false) : child().treeDualtype(sjTree.child())) : false;
	}

	public SJType subsume(SJType sjType) throws SemanticException
	{
		return treeSubsume(sjType);
	}

	public SJType treeSubsume(SJType sjTree) throws SemanticException
	{
		SJType ours = child();
		SJType theirs = sjTree.child();

		if (ours == null)
		{
			if (theirs != null)
			{
				throw new SemanticException("Not subsumable: " + this + ", " + sjTree);
			}

			return nodeSubsume(sjTree);
		}
		else
		{
			if (theirs == null)
			{
				throw new SemanticException("Not subsumable: " + this + ", " + sjTree);
			}

			return nodeSubsume(sjTree).child(ours.subsume(theirs));
		}
	}

	public boolean wellFormed()
	{
		if (!(this instanceof SJBeginType && typeSystem().wellFormedRecursions(this)))
		{
			return false;
		}

		return (child() == null) ? true : child().treeWellFormed();
	}

	public boolean treeWellFormed()
	{
		if (!nodeWellFormed())
		{
			return false;
		}

		return (child() == null) ? true : child().treeWellFormed();
	}

	public SJType copy()
	{
		return treeClone();
	}

	public SJType treeClone()
	{
		return (child() == null) ? nodeClone() : nodeClone().child(child().treeClone());
	}

	public String toString()
	{
		return treeToString();
	}

	public String treeToString()
	{
		return (child() == null) ? nodeToString() : nodeToString() + SJ_STRING_SEPARATOR + child().treeToString();
	}

	public String translate(Resolver c) // HACK: FIXME.
	{
		return null;
	}

	public boolean equalsImpl(TypeObject to) // Used instead of standard Object equals.
	{
		if (to instanceof SJType)
		{
			return typeEquals((SJType) to);
		}
		else
		{
			return false;
		}
	}

	/*public int hashCode()
	{
		return toString().hashCode(); // Is this an acceptable design?
	}*/
}
