/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.util;

import java.util.*;

import sj.types.typeobjects.sesstypes.SJType;

/**
 * Maps Strings (session socket variable names) to SJTypes.
 */
public class SJMap
{
	private Map<String, SJType> map;

	public SJMap()
	{
		map = new LinkedHashMap<String, SJType>(); // Preserves insertion order (most uses should be for add-only or immutable collections).
	}

	public Set<String> keySet()
	{
		return map.keySet();
	}

	public boolean containsKey(String key)
	{
		return keySet().contains(key);
	}

	public SJType get(String key)
	{
		return map.get(key);
	}

	public SJType get(int index) throws IndexOutOfBoundsException
	{
		int i = 0;

		for (String key : keySet())
		{
			if (i == index)
			{
				return get(key);
			}
			else
			{
				i++;
			}
		}

		throw new IndexOutOfBoundsException("Index out of bounds: " + index);
	}

	public SJType put(String key, SJType t)
	{
		map.put(key, t);

		return t;
	}

	public void remove(String key)
	{
		map.remove(key);
	}

	public String toString()
	{
		return map.toString();
	}
}
