/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.util;

import polyglot.ast.*;
import polyglot.frontend.Job;
import polyglot.types.*;
import polyglot.visit.*;

/**
 * SJTypeBuilder builds the message types for send and receive session type elements.
 */
public class SJTypeBuilder
{
	private Job job;
	private TypeSystem ts;
	private NodeFactory nf;

	public SJTypeBuilder(Job job, TypeSystem ts, NodeFactory nf) // Ok to fix the job?
	{
		this.job = job;
		this.ts = ts;
		this.nf = nf;
	}

	// Hacky: won't work for AST substitutions that need the type builder to update the (outer) context e.g. SJProcedureDecl parsing.
	public Node buildTypes(Context context, Node n) throws SemanticException
	{
		//Context context = (Context) context.copy(); // Unnecessary to copy. (Copying will break translations that update the outer context?) // Copying is shallow. Problems with inserting LocalDecls due to AmbiguityRemover updating the (shared) context and TypeChecker seeing those updates (thinks local multiply declared).

		TypeBuilder tb = new TypeBuilder(job, ts, nf).pushContext(context);
		AmbiguityRemover ar = (AmbiguityRemover) new AmbiguityRemover(job, ts, nf, true, true).context(context);
		TypeChecker tc = (TypeChecker) new TypeChecker(job, ts, nf).context(context);

		// FIXME: should also do ConstantChecker.

		n = n.visit(tb);
		n = n.visit(ar);
		n = n.visit(tc);

		return n;
	}
}
