/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.visit;

import java.util.*;

import polyglot.ast.*;
import polyglot.frontend.Job;
import polyglot.types.*;
import polyglot.visit.ErrorHandlingVisitor;
import polyglot.visit.NodeVisitor;

import sj.SJConstants;
import sj.ast.*;
import sj.ast.proceduredecls.*;
import sj.ast.sessformals.*;
import sj.util.*;

// HACK: also translates the String argument of recurse operations.
public class SJProcedureDeclParser extends ErrorHandlingVisitor
{
	private SJNodeFactory sjnf = (SJNodeFactory) nodeFactory();

	//private SJTypeBuilder sjtb = new SJTypeBuilder(job(), typeSystem(), sjnf);

	public SJProcedureDeclParser(Job job, TypeSystem ts, NodeFactory nf)
	{
		super(job, ts, nf);
	}

	protected Node leaveCall(Node old, Node n, NodeVisitor v) throws SemanticException
	{
		if (n instanceof ProcedureDecl)
		{
			if (n instanceof SJMethodDecl) // Parsed by actual parser.
			{
				if (!((SJMethodDecl) n).flags().isPrivate()) // The SJThread run method has void return type, so it isn't parsed until this pass, and so it is not detected by this check.
				{
					throw new SemanticException("Non-private session-methods not done yet: " + n);
				}

				/*boolean foundSJArg = false; // Until runtime (state tracking) has been extending with contexts that support multiple sessions. // No, each session currently corresponds to an individual socket: so each session is being tracked separately, and context objects are per session. // But maybe later need to extend runtime state tracking to support session multiplexing over a single socket (or just duplicate context management for each session per socket).

				for (Object formal : ((SJMethodDecl) n).formals())
				{
					if (formal instanceof SJFormal)
					{
						if (foundSJArg)
						{
							throw new SemanticException("Polyadic session procedures not done yet: " + n);
						}

						foundSJArg = true;
					}
				}*/
			}
			else
			{
				n = parseProcedureDecl((ProcedureDecl) n);
			}
		}
		if (n instanceof Call) // HACK.
		{
			Call call = (Call) n;

			if (call.name().equals(SJConstants.SJ_SYNTAX_RECURSE))
			{
				List args = call.arguments();

				if (args.size() == 1)
				{
					Expr arg = (Expr) args.get(0);

					if (!(arg instanceof StringLit))
					{
						String lab = arg.toString();

						List<Expr> foo = new LinkedList<Expr>();

						foo.add(sjnf.StringLit(arg.position(), lab.substring(0, lab.length() - "{amb}".length()))); // HACK.

						n = call.arguments(foo);
					}
				}
			}
		}

		return n;
	}

	private ProcedureDecl parseProcedureDecl(ProcedureDecl pd) throws SemanticException
	{
		List<Formal> formals = (List<Formal>) pd.formals();

		//boolean isSJProcedure = false; // Until runtime (state tracking) has been extending with contexts that support multiple sessions.

		for (Object formal : formals)
		{
			if (formal instanceof SJFormal)
			{
				/*if (isSJProcedure)
				{
					throw new SemanticException("Polyadic session procedures not done yet: " + pd);
				}*/

				// List throwTypes = md.throwTypes(); // Don't need to check here - resolved by a combination of standard exception checking and session type checking (of scopes). // Also, don't know if can actually do this, since this pass now comes before standard type building.

				if (pd instanceof ConstructorDecl)
				{
					throw new SemanticException("SJ constructors not done yet: " + pd);
				}
				else //if (pd instanceof MethodDecl) // Private flag checked in SJTypeBuilder.
				{
					MethodDecl md = (MethodDecl) pd;

					pd = sjnf.SJMethodDecl(md.position(), md.flags(), md.returnType(), md.id(), formals, (List<TypeNode>) md.throwTypes(), md.body());

					SJCompilerUtils.debugPrintln("Parsed: " + pd);

					//isSJProcedure = true;
				}

				break;
			}
		}

		return pd;
	}
}
