/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.visit.contexts.sesscontexts;

import java.util.Set;

import sj.types.typeobjects.sesstypes.*;
import sj.util.*;

// Session method scopes are like session-try scopes without catch handlers (exceptions must be propagated back).
public class SJMethodContext_c extends SJSessionTryContext_c implements SJMethodContext
{
	private String name;
	private SJMap sjFormals;
	private SJType sjReturnType; // Null if not a session return type.

	public SJMethodContext_c(String name, Set<String> sockets, SJMap sjFormals, SJType sjReturnType)
	{
		super(sockets); // Only registered inactive sockets can be passed in (session methods are an extension of the calling session-try context - that's the difference between session methods and session threads, the latter define new separate session-try scopes).

		this.name = name;
		this.sjFormals = sjFormals;
		this.sjReturnType = sjReturnType;

		for (String session : sjFormals.keySet())
		{
			SJType sjType = sjFormals.get(session);

			if (!(sjType instanceof SJBeginType)) // SJMaps preserve insertion order.
			{
				openSession(session, sjType);

				SJCompilerUtils.debugPrintln("Opened: `" + session + "' " + sjType);
			}
		}
	}

	public String name()
	{
		return name;
	}

	public SJMap sjFormals()
	{
		return sjFormals;
	}

	public SJType sjReturnType()
	{
		return sjReturnType;
	}
}
