/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.runtime.net;

import java.io.IOException;
import java.net.*;

import sj.runtime.*;

public final class SJFServerSocket extends SJServerSocket
{
	public static final SJFServerSocket create(SJProtocol protocol, int port) throws IOException
	{
		return new SJFServerSocket(protocol, port);
	}

	private SJFServerSocket(SJProtocol protocol, int port) throws IOException
	{
		super(protocol, port);
	}

	public final SJFSocket accept() throws SJIOException, SJIncompatibleSessionException
	{
		checkClassLoadingParameters();

		Socket socket = null;

		try
		{
			socket = getServer().accept();
			socket.setTcpNoDelay(SJSocket.TCP_NO_DELAY);
		}
		catch (IOException ioe)
		{
			throw new SJIOException(ioe);
		}

		SJFSocket fsocket = new SJFSocket(getSJType(), getProtocol(), socket); // Session type cloned (if needed) in socket constructor.

		// Probably faster to use these setters than make a new parameters object?
		fsocket.setRemoteClassLoading(remoteClassLoadingEnabled());
		fsocket.setEagerClassVerification(eagerClassVerificationEnabled());
		fsocket.setEagerClassDownloading(eagerClassDownloadingEnabled());
		fsocket.setCodebase(getCodebase());

		fsocket.acceptIOStreams();

		//SJRuntimeUtils.debugPrintln("Accepted connection on " + getLocalAddress().getPort() + " from: " + socket.getInetAddress().getHostName() + ":" + socket.getPort()); // Server-accepted sockets have the same local port as the server, traffic is identified by the peer address (and port).

		fsocket.init();
		fsocket.isActive(true);

		if (SJFSocket.RUNTIME_STATE)
		{
			fsocket.state.accept();

			//SJRuntimeUtils.debugPrintln("Current state: " + fsocket.state.currentState());
		}

		return fsocket; // Original requestor value defaults to false (so we don't need to set).
	}
}
