/*
 * This file is part of SessionJ.
 *
 * Copyright (C) Raymond Hu, Imperial College London.
 *
 */

package sj.runtime.net;

import java.io.Serializable;
import java.net.*;

import sj.runtime.*;

public class SJSocketParameters implements Serializable
{
	public static final long serialVersionUID = 1L;

	private boolean origreq = SJSocket.NOT_ORIGINAL_REQUESTOR;
	private boolean remoteClassLoading = false;
	private boolean eagerVerification = false;
	private boolean eagerDownloading = false;
	private URL codebase = null; // Optimise as a String for Serialization?

	public SJSocketParameters()
	{

	}

	public SJSocketParameters(boolean origreq, boolean remoteClassLoading, boolean eagerVerification, boolean eagerDownloading, URL codebase)
	{
		this.remoteClassLoading = remoteClassLoading;
		this.eagerVerification = eagerVerification;
		this.eagerDownloading = eagerDownloading;
		this.codebase = codebase;
	}

	public final boolean isOriginalRequestor()
	{
		return origreq;
	}

	public final URL getCodebase()
	{
		return codebase;
	}

	public final boolean remoteClassLoadingEnabled()
	{
		return remoteClassLoading;
	}

	public final boolean eagerClassDownloadingEnabled()
	{
		return eagerDownloading;
	}

	public final boolean eagerClassVerificationEnabled()
	{
		return eagerVerification;
	}

	public final void setOriginalRequestor(boolean origreq)
	{
		this.origreq = origreq;
	}

	public final void setRemoteClassLoading(boolean enabled) throws SJIOException
	{
		this.remoteClassLoading = enabled;
	}

	public final void setEagerClassDownloading(boolean eager) throws SJIOException
	{
		this.eagerDownloading = eager;
	}

	public final void setEagerClassVerification(boolean eager) throws SJIOException
	{
		this.eagerVerification = eager;
	}

	public final void setCodebase(URL codebase) throws SJIOException
	{
		this.codebase = codebase;
	}

	public final void setCodebase(String codebase) throws SJIOException
	{
		try
		{
			setCodebase(new URL(codebase));
		}
		catch (MalformedURLException mue)
		{
			throw new SJIOException(mue);
		}
	}

	public final String toString()
	{
		return "<isOriginalRequestor=" + isOriginalRequestor() + ", remoteClassLoading=" + remoteClassLoadingEnabled() + ", eagerClassDownloading=" + eagerClassDownloadingEnabled() + ", eagerClassVerification=" + eagerClassVerificationEnabled() + ", codebase=" + getCodebase() + ">";
	}
}
