//$ bin/sjc tests/src/sesssock/SJSSocket.sj -d tests/classes/
//$ bin/sj -cp tests/classes/ sesssock.SJSSocket 

package sesssock;

import java.io.*;
import java.net.*;
import java.util.*;

import sj.types.typeobjects.sesstypes.*;

import sj.runtime.*;
import sj.runtime.net.*;
import sj.runtime.util.*;

public class SJSSocket extends SJSocket
{
	private protocol ourSide
	{
		begin
		.rec X[
			!{
				SENDOBJ: !<Object>.#X,
				//SENDINT: !<int>.#X,
				//SENDBOOL: !<boolean>.#X,
				//SENDBYTE: !<byte>.#X,
				CLOSE:
			}
		]
	}

	private final SJType protocolType;

	private List sessops = new LinkedList();

	public static SJSSocket create(SJServerAddress channel)
	{
		return new SJSSocket(channel);
	}

	private SJSSocket(SJServerAddress channel)
	{
		setChannel(channel);

		this.protocolType = decodeSJType(channel.getProtocol()); 
	}

	protected SJSSocket(SJType sjType, SJProtocol encoded, Socket socket)
	{
		setChannel(SJServerAddress.create(encoded, (InetSocketAddress) socket.getRemoteSocketAddress()));

		setSocket(socket); 

		this.protocolType = sjType; 
	}

	public void request() throws UnknownHostException, SJIOException, SJIncompatibleSessionException
	{
		SJSocket s = SJFSocket.create(getChannel());
		
		try (s)
		{
			s.request();
			
			s.recursion(X)
			{
				SessOp sessop = null;
			
				synchronized (sessops)
				{
					while (sessops.isEmpty())
					{
						sessops.wait();
					}

					sessop = (SessOp) sessops.remove(0);
				}
					
				if (sessop.isObject())
				{
					s.outbranch(SENDOBJ)
					{
						s.send(sessop.getObject());

						s.recurse(X);
					}
				}
				else //if (sessop.isClose())
				{
					s.outbranch(CLOSE)
					{
					
					}
				}
			}
		}
		finally
		{
		
		}
	}

	public void send(Object obj) throws SJIOException
	{
		synchronized (sessops)
		{
			sessops.add(new SessOp(obj));
			
			sessops.notifyAll();
		}
	}

	public void sendBoolean(boolean v) throws SJIOException
	{
		throw new UnsupportedOperationException("sendSession not done yet.");
	}	
	
	public void sendInt(int v) throws SJIOException
	{
		throw new UnsupportedOperationException("sendSession not done yet.");
	}		

	public void sendByte(byte v) throws SJIOException
	{
		throw new UnsupportedOperationException("sendSession not done yet.");
	}	

	public Object receive() throws SJIOException, ClassNotFoundException
	{
		throw new UnsupportedOperationException("sendSession not done yet.");
	}		
	
	public boolean receiveBoolean() throws SJIOException
	{
		throw new UnsupportedOperationException("sendSession not done yet.");
	}		
	
	public int receiveInt() throws SJIOException
	{
		throw new UnsupportedOperationException("sendSession not done yet.");
	}		

	public byte receiveByte() throws SJIOException
	{
		throw new UnsupportedOperationException("sendSession not done yet.");
	}	

	public void sendSession(SJSocket s, String encoded) throws SJIOException
	{
		throw new UnsupportedOperationException("sendSession not done yet.");
	}		
	
	public void sendServerAddress(SJServerAddress sjsa) throws SJIOException
	{
		throw new UnsupportedOperationException("sendServerAddress not done yet.");
	}	

	public SJSocket receiveSession(String encoded) throws SJIOException
	{
		throw new UnsupportedOperationException("receiveSession not done yet.");
	}
	
	public SJServerAddress receiveServerAddress(String encoded) throws SJIOException
	{
		throw new UnsupportedOperationException("receiveServerAddress not done yet.");
	}	

	public String outlabel(String lab) throws SJIOException
	{
		throw new UnsupportedOperationException("outlabel not done yet.");
	}
	
	public String inlabel() throws SJIOException
	{
		throw new UnsupportedOperationException("inlabel not done yet.");
	}
	
	public boolean outsync(boolean bool) throws SJIOException
	{
		throw new UnsupportedOperationException("outsync not done yet.");
	}
	
	public boolean insync() throws SJIOException
	{
		throw new UnsupportedOperationException("insync not done yet.");
	}
	
	public boolean recursionFalse(String lab) throws SJIOException
	{
		throw new UnsupportedOperationException("recursionFalse not done yet.");
	}
	
	public boolean recursionTrue(String lab) throws SJIOException
	{
		throw new UnsupportedOperationException("recursionTrue not done yet.");
	}

	protected void close() /*throws SJIOException*/
	{
	
	}

	public SJType protocolType()
	{
		return protocolType;
	}
	
	public SJType currentState()
	{
		throw new UnsupportedOperationException("SJSSocket does not record state.");
	}
}

class SessOp
{
	private static final int OBJECT = 1;
	private static final int INT = 2;
	private static final int BOOLEAN = 3;
	private static final int BYTE = 4;
	private static final int CLOSE = 5;
	
	private int type;
	private Object message;
	
	public SessOp(Object obj)
	{
		type = OBJECT;
		message = obj;
	}
	
	public SessOp(int v)
	{
		type = INT;
		message = new Integer(v);
	}

	public SessOp(boolean v)
	{
		type = BOOLEAN;
		message = new Boolean(v);
	}
	
	public SessOp(byte v)
	{
		type = BYTE;
		message = new Byte(v);
	}	
	
	public boolean isObject()
	{
		return type == OBJECT;
	}

	public boolean isInt()
	{
		return type == INT;
	}
	
	public boolean isBoolean()
	{
		return type == BOOLEAN;
	}
	
	public boolean isByte()
	{
		return type == BYTE;
	}	

	public boolean isClose()
	{
		return type == CLOSE;
	}	
	
	public Object getObject()
	{
		return message;
	}

	public int getInt()
	{
		return ((Integer) message).intValue();
	}

	public boolean getBoolean()
	{
		return ((Boolean) message).booleanValue();
	}	
	
	public byte getByte()
	{
		return ((Byte) message).byteValue();
	}
}
